## copy the labels to a single array
from whacc import utils
import numpy as np
import h5py
from natsort import natsorted
import copy

label_files = natsorted(utils.get_files("/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/", "*_labels.npy"))
label_files = utils.lister_it(label_files, keep_strings=None, remove_string='curated')

all_labels = []
for k in label_files:
    all_labels.append(np.load(k)[0])

h5_file = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
with h5py.File(h5_file, 'r') as hf:
    frame_nums = copy.deepcopy(hf['trial_nums_and_frame_nums'][1, :])

new_labels = np.zeros(int(np.sum(frame_nums))).astype(int)
for ii, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
    print(ii)
    for k in all_labels[ii]:
        new_labels[i1:i2][int(k-1)] = 1


h5_file = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
with h5py.File(h5_file, 'r+') as hf:
    hf['labels'][:] = new_labels

##################
# plot random touch onsets
import matplotlib.pyplot as plt
from whacc import image_tools

all_onsets = np.where(np.diff(new_labels) == 1)[0]+1
add2 = 2
h5_file = "/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5"
num_plots = 10
with h5py.File(h5_file, 'r') as hf:
    for k in np.random.choice(all_onsets, num_plots):
        plt.figure()
        plt.imshow(image_tools.img_unstacker(hf['images'][k-add2 : k+add2+1], num_frames_wide=1+add2*2))
        print(k-add2,  k+add2+1, k)


#######
# make a new reduced sized only near touches file with only one image deep
from whacc import utils
from whacc import image_tools
import copy
import h5py
f = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000.h5'
f2 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single.h5'

h5c = image_tools.h5_iterative_creator(f2, overwrite_if_file_exists=True, color_channel=False)

with h5py.File(f, 'r') as h:
    print(len(h['labels'][:]))
    h_cont = copy.deepcopy(h['labels'][:])
utils.create_master_dataset(h5c, [f], [h_cont], borders=80, max_pack_val=100)






# everything you need to make the below plot is here
# plot the histogram for the comparing the error types
a = []
for k in list(itertools.permutations(range(4), 2)):
  a.append(analysis.error_analysis(h_cont_1[k[1], :], h_cont_1[k[0], :], frame_num_array=frame_num_array.astype(int)))

def get_error_str_list(a):
  all_errors = []
  for k in a.error_neg:
    all_errors.append(a.type_list[k])
  for k in a.error_pos:
    all_errors.append(a.type_list[k])
  return all_errors

a_dict = dict()
a_dict['Count'] = []
a_dict['Error type'] = []
a_dict['pairs'] = []
# a_dict['count'] = []
for i, k in enumerate(list(itertools.permutations(range(4), 2))):
  if 3 in k:
    out = get_error_str_list(a[i])
    out = out+['append', 'deduct', 'ghost', 'join', 'miss', 'split']
    keys, counts = np.unique(out, return_counts=True)
    counts = counts-1
    a_dict['Count'] += list(counts)
    a_dict['Error type'] += list(keys)
    a_dict['pairs'] += list(np.tile('pairs '+str(k), len(keys)))

for i, k in enumerate(list(itertools.permutations(range(4), 2))):
  if 3 not in k:
    out = get_error_str_list(a[i])
    out = out+['append', 'deduct', 'ghost', 'join', 'miss', 'split']
    keys, counts = np.unique(out, return_counts=True)
    counts = counts-1
    a_dict['Count'] += list(counts)
    a_dict['Error type'] += list(keys)
    a_dict['pairs'] += list(np.tile('pairs '+str(k), len(keys)))


import pandas as pd
import datetime
import seaborn as sns

df = pd.DataFrame.from_dict(a_dict)
plt.figure(figsize=(15, 10))
# plt.ylim([0, 300])
# plt.yscale("log")
sns.barplot(x="Error type", hue="pairs", y="Count", data=df)

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""1$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$make a very small onset and offset extracot!!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""
som enotes might have to do this for eeach weird extracted lag and lad diff...check this
"""

from whacc import utils, image_tools, analysis
import numpy as np
import h5py


all_h5s_imgs = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_using_pole_tracker.h5']
h_cont = image_tools.get_h5_key_and_concatenate(all_h5s_imgs, 'labels')
h5c = image_tools.h5_iterative_creator('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_5border_single_frame_onset_offset_for_aug.h5',
                                           overwrite_if_file_exists = True,
                                           color_channel = False)

h_cont_onset_offset = np.append(0, np.abs(np.diff(h_cont))) # to get jsut the onsets and ofsets not necesarilly the middles
utils.create_master_dataset(h5c, all_h5s_imgs, [h_cont_onset_offset], borders=5, max_pack_val=100)

utils.print_h5_keys(h5c.h5_full_file_name)
corrected_labels = []
with h5py.File(h5c.h5_full_file_name, 'r+') as h:
    for k in h['inds_extracted'][:]:
        corrected_labels.append(h_cont[k])
    h['labels'][:] = corrected_labels

tmp1 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_5border_single_frame_onset_offset_for_aug.h5'
a = analysis.pole_plot(tmp1, true_val=image_tools.get_h5_key_and_concatenate([tmp1]))
a.next()

"""2$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$ make the different types of h5 images and labels $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
base_dir_all_h5s = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/single_frame/'
utils.make_all_H5_types(base_dir_all_h5s)  # auto generate all the h5 types using a single set of flat (no color) image H5
utils.make_alt_labels_h5s(base_dir_all_h5s)  # auto generate the different types of labels

"""3$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$trim and correct the edges of the small subset $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
to_convert = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/regular/AH0000x000000_5border_single_frame_onset_offset_for_aug_regular.h5'
to_convert = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag_diff/AH0000x000000_5border_single_frame_onset_offset_for_aug_3lag_diff.h5'
to_convert = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag/AH0000x000000_5border_single_frame_onset_offset_for_aug_3lag.h5'
utils.print_h5_keys(to_convert)
xxx = to_convert.split('.h5')[0] + '_remove_2_edges' +'.h5'
h5c = image_tools.h5_iterative_creator(xxx,
                                       overwrite_if_file_exists = True,
                                       color_channel = True)
from tqdm import tqdm
border_bash = 2
with h5py.File(to_convert, 'r') as h:
    multiplier = h['multiplier'][:]
    frame_nums = []
    for i1, i2 in tqdm(utils.loop_segments(h['frame_nums'][:])):
        i1 += border_bash
        i2 -= border_bash
        h5c.add_to_h5(h['images'][i1:i2], h['labels'][i1:i2])
        frame_nums.append(i2-i1)
with h5py.File(h5c.h5_full_file_name, 'r+') as h:
    h.create_dataset('frame_nums', shape=np.shape(frame_nums), data=frame_nums)
    # print(h['multiplier'][:])
    # h.create_dataset('multiplier', shape=np.shape(multiplier), data=multiplier)

a = image_tools.get_h5_key_and_concatenate([to_convert], key_name='frame_nums')
b = image_tools.get_h5_key_and_concatenate([xxx], key_name='frame_nums')
from whacc import analysis

labels = image_tools.get_h5_key_and_concatenate([xxx], key_name='labels')
c = analysis.pole_plot(xxx, true_val=labels)
c.plot_it()


"""3$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ make all the augmented files  $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""


from whacc import utils
from whacc import image_tools
from keras.preprocessing.image import ImageDataGenerator
import h5py
import matplotlib.pyplot as plt
from tqdm import tqdm
from imgaug import augmenters as iaa  # optional program to further augment data
# h5_subset_file = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag/AH0000x000000_5border_single_frame_onset_offset_for_aug_3lag_remove_2_edges.h5'
for h5_subset_file in utils.get_files('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/', '*remove_2_edges.h5'):
    datagen = ImageDataGenerator(rotation_range=360,  #
                                 width_shift_range=.1,  #
                                 height_shift_range=.1,  #
                                 shear_range=.00,  #
                                 zoom_range=.25,
                                 brightness_range=[0.2, 1.2])  #

    with h5py.File(h5_subset_file, 'r') as hf:
        test_img = hf['images'][2]  # grab a single image
    num_aug = 20

    aug_ims, _ = image_tools.augment_helper(datagen, num_aug, 0 ,test_img, -1)  # make 99 of the augmented images and output 1 of th original
    # '-1' is the label associate ith the image, we dont care becasue we just want the iamge so it doenst matter what it is here

    # plt.figure(figsize=[5, 5])
    # plt.imshow(image_tools.img_unstacker(aug_ims, 10), )
    # plt.show()

    gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
    aug_ims_2 = gaussian_noise.augment_images(aug_ims)
    # plt.figure(figsize=[10, 10])
    # plt.imshow(image_tools.img_unstacker(aug_ims_2, 10))
    # plt.show()

    num_aug = 1
    for ii in range(10):
        # once we are happy with our augmentation process we can make an augmented H5 file using class
        new_H5_file = h5_subset_file.split('.')[0] + '_AUG_' + str(ii) + '.h5' # create new file name based on teh original H5 name
        h5creator = image_tools.h5_iterative_creator(new_H5_file, overwrite_if_file_exists=True,
                                                     close_and_open_on_each_iteration=True, color_channel=True)
        utils.get_class_info(h5creator)
        with h5py.File(h5_subset_file, 'r') as hf:
            for image, label in zip(tqdm(hf['images'][:]), hf['labels'][:]):
                aug_img_stack, labels_stack = image_tools.augment_helper(datagen, num_aug, 0, image, label)
                aug_img_stack = gaussian_noise.augment_images(aug_img_stack) # optional
                h5creator.add_to_h5(aug_img_stack[:, :, :, :], labels_stack)

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  transfer the frame nums to the aug data    $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

bd_all = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/regular/',
          '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag_diff/',
          '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag/']
for bd in bd_all:
    h5_to_split_list = utils.get_files(bd, '*edges_AUG_*')
    frame_num_src = utils.get_files(bd, '*2_edges.h5')[0]
    for dest in h5_to_split_list:
        utils.copy_h5_key_to_another_h5(frame_num_src, dest, 'frame_nums', 'frame_nums')


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  combine them into training and validation    $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

from whacc import image_tools
from whacc import utils

bd_all = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/regular/',
          '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag_diff/',
          '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag/']

for bd in bd_all:
    h5_to_split_list = utils.get_files(bd, '*edges_AUG_*')
    image_tools.split_h5_loop_segments(h5_to_split_list, [7, 3], [bd+'train', bd+'val'], chunk_size=1000, add_numbers_to_name=False,
                 disable_TQDM=False, set_seed = 0, color_channel=True)



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$  make a final combination of the the aug and other files    $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

def label_naming_shorthand_dict(name_key=None):
    label_naming_shorthand_dict = {
        '[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)': 'on-off_set_and_one_after',
        '[0, 1, 2, 3]- (no touch, touch, onset, offset': 'on-off_set',
        '[0, 1, 2]- (no event, onset, offset)': 'only_on-off_set',
        '[0, 1]- (no touch, touch)': 'regular',
        '[0, 1]- (not offset, offset)': 'only_offset',
        '[0, 1]- (not onset, onset)': 'only_onset',
        '[0, 1, 3, 4]- (no touch, touch, one after onset, offset)': 'overlap_whisker_on-off'}
    if name_key is None:
        return label_naming_shorthand_dict
    else:
        return label_naming_shorthand_dict[name_key]
def copy_over_new_labels(label_key_name, image_h5_list, label_h5_list):
    label_key_shorthand = label_naming_shorthand_dict(label_key_name)
    for img_src, lab_src in zip(image_h5_list, label_h5_list):
        utils.copy_h5_key_to_another_h5(lab_src, img_src, label_key_name, 'labels')

from whacc import image_tools
from whacc import utils
bd_all = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/regular/',
          '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag_diff/',
          '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/for_aug/3lag/']

to_combine = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/regular',
              '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/3lag_diff/',
              '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/3lag/']

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
""" make sure the labels are regular before combining them with the aug set"""
k = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/ALT_LABELS'
alt_labels = utils.get_files(k, '*.h5')

for k in to_combine:
    a = utils.get_files(k, '*.h5')
    a = utils.lister_it(a, remove_string = 'zip')
    print(a)
    copy_over_new_labels('[0, 1]- (no touch, touch)', a, alt_labels)

for bd, cmb in zip(bd_all, to_combine):
    l1 = utils.get_files(bd, '*train*')
    l2 = utils.get_files(cmb, '*train*')
    l2 = utils.lister_it(l2, remove_string = 'zip')
    image_tools.split_h5_loop_segments(l1+l2, [1], [bd+'train_all'], chunk_size=1000, add_numbers_to_name=False,
                 disable_TQDM=False, set_seed = 0, color_channel=True)

    l1 = utils.get_files(bd, '*val*')
    l2 = utils.get_files(cmb, '*val*')
    l2 = utils.lister_it(l2, remove_string = 'zip')
    image_tools.split_h5_loop_segments(l1+l2, [1], [bd+'val_all'], chunk_size=1000, add_numbers_to_name=False,
                 disable_TQDM=False, set_seed = 0, color_channel=True)

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$# finally after copying them to this directory we can make the labels with any of the data directories$$$$$$$"""
# be careful with this cause if the regular frames is not the normal one then your labels will be all screwed up
# need to run below code above first
'''
for k in to_combine:
    a = utils.get_files(k, '*.h5')
    a = utils.lister_it(a, remove_string = 'zip')
    print(a)
    copy_over_new_labels('[0, 1]- (no touch, touch)', a, alt_labels)
'''
utils.make_alt_labels_h5s('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/regular')

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$# update all the labels in case you create a new label """
utils.make_alt_labels_h5s('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/small_h5s/data/single_frame')
utils.make_alt_labels_h5s('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/single_frame')
utils.make_alt_labels_h5s('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/small_h5s/single_frame')
utils.make_alt_labels_h5s('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/10_percent_holy_set/single_frame')
utils.make_alt_labels_h5s('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/full_holy_set/single_frame')

"""
USE THIS AFTER CREATING AUG IMAGES TO TRANSFER FRAME NUMS
all_aug = utils.get_files('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/', 'AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_*.h5')
frame_num_src = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug.h5'
for k in all_aug:
     utils.copy_h5_key_to_another_h5(frame_num_src, k, 'frame_nums', 'frame_nums')

"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
from whacc import utils
all_h5s = utils.get_h5s('/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/finished_contacts/')
all_h5s_imgs = utils.get_h5s('/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/H5_data/')
h_cont = utils._get_human_contacts_(all_h5s)
h5c = image_tools.h5_iterative_creator('/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/holy_set_80_border_single_frame.h5',
                                       overwrite_if_file_exists = True,
                                       color_channel = False)
utils.create_master_dataset(h5c, all_h5s_imgs, h_cont, borders = 80, max_pack_val = 100)


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ split data based on frame num segments!!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

from whacc import image_tools

bd = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/'
h5_to_split_list = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5']
image_tools.split_h5_loop_segments(h5_to_split_list, [7, 3], [bd+'train', bd+'val'], chunk_size=1000, add_numbers_to_name=False,
             disable_TQDM=False, set_seed = 0, color_channel=False)



bd = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/'
h5_to_split_list = ['/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_80border_single_frame.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_9.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_8.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_7.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_6.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_5.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_4.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_3.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_2.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_1.h5',
                    '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/AH0000x000000_3border_single_frame_onset_offset_for_aug_AUG_0.h5']

image_tools.split_h5_loop_segments(h5_to_split_list, [7, 3], [bd+'train', bd+'val'], chunk_size=1000, add_numbers_to_name=False,
             disable_TQDM=False, set_seed = 0, color_channel=False)

a = bd+'val'
a = a+'.h5'
a = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/regular/train_regular.h5'
a = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/single_frame/train.h5'
# a = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/3lag_diff/val_3lag_diff.h5'
a = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border/data/3lag/val_3lag.h5'
import h5py
with h5py.File(a, 'r') as h:
    print(h['images'][:].shape)
    print(h['labels'][:].shape)
    print(sum(h['frame_nums'][:]))

a = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/ALT_LABELS/val_ALT_LABELS.h5'
with h5py.File(a, 'r') as h:
    print(h['[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)'][:].shape)
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  make the mini holy test set for the   $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
from whacc import image_tools, utils
bd = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/10_percent_holy_set/data/single_frame/'
h5_to_split_list = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/full_holy_set/single_frame/holy_set_80_border_single_frame.h5'
image_tools.split_h5_loop_segments([h5_to_split_list], [1, 9], [bd+'holy_test_set_10_percent', [None]], chunk_size=1000, add_numbers_to_name=False,
             disable_TQDM=False, set_seed = 0, color_channel=False)
utils.print_h5_keys('/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/test_data/10_percent_holy_set/data/single_frame/holy_test_set_10_percent.h5')


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$  plot that shit   $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

import matplotlib.pyplot as plt
a = info_dict['h5_train']
labels = image_tools.get_h5_key_and_concatenate([a])
bplt = analysis.pole_plot(a, true_val = labels)
points_of_interest = np.where(np.diff(labels)==1)[0]
cnt = -1


cnt+=1
bplt.current_frame = points_of_interest[cnt]-4
bplt.plot_it()
plt.ylim([-.5, 6])







from whacc import image_tools
from whacc import analysis

xx = '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/model_testing/all_data/all_models/regular_80_border_aug_0_to_9/data/regular/train_regular.h5'
a = analysis.pole_plot(xx)
a.plot_it()
fn = image_tools.get_h5_key_and_concatenate([xx], 'frame_nums')


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$  transfer all model and folders form google cloud machine to google bucket  $$$$$$$$$$$$$$$$$$$$$$$$"""

from google.cloud import storage # !pip install --upgrade google-cloud-storage
import os
from whacc import utils
%cd "/home/jupyter"

client = storage.Client()
storage_client = storage.Client.from_service_account_json('phils-whisker-contacts-7eea4efe3802.json')
bucket = client.get_bucket('whacc_multi_model_test_data')

def transfer_data(bucket):
    blobs=list(bucket.list_blobs(prefix='SAVED_MODELS'))
    cloud_folders = list(set([os.path.dirname(str(k.name)) for k in blobs]))
    a = utils.get_files('model_testing', '*info_dict.json')
    for k in a:
        to_copy = os.path.dirname(k)
        if not any([to_copy in kk for kk in cloud_folders]):
            print(to_copy)
            dest = 'gs://whacc_multi_model_test_data/'+'SAVED_MODELS/'+to_copy
            !gsutil -m cp -r $to_copy $dest
transfer_data(bucket)
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$  reduce to single frames  $$$$$$$$$$$$$$$$$$$$$$$$"""

from pathlib import Path
from whacc import utils, image_tools
import os
from tqdm import tqdm

h5files = utils.get_h5s('/Users/phil/Dropbox/Colab data/H5_data')
for k1 in h5files:
    # print(k.split('__')[0])
    k = '-'.join(k1.split('__'))
    k = k.split('-')[0]
    k = os.path.dirname(k) + os.sep + 'single_frame' + os.sep + os.path.basename(k) + '.h5'
    Path(os.path.dirname(k)).mkdir(parents=True, exist_ok=True)
    utils.reduce_to_single_frame_from_color(k1, k)

utils.make_all_H5_types('/Users/phil/Dropbox/Colab data/H5_data/single_frame/')

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$run in google colab$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$  needs search_sequence_numpy from colab, adds all the correct labels to the H5  $$$$$$$$$$$$$$$$$$$$$$$$"""
"""used to make the final 'real contats, based on average of human and some basic editing """
from google.colab import drive
import numpy as np
import h5py
import os
from whacc import utils

drive.mount('/content/gdrive')

base_dir = '/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/'
h5_list_to_write = utils.get_h5s(base_dir)
all_h5s = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/',
                        print_h5_list=False)
h_cont, h_names = utils._get_human_contacts_(all_h5s)
print('--------\npercent fully agree\n--------')
for k in h_cont:
    a = np.mean(np.mean(k, axis=0) == 1)
    b = np.mean(np.mean(k, axis=0) == 0)
    print(np.round(a + b, 4))


def add_to_h5(h5_file, key, values, overwrite_if_exists=False):
    all_keys = utils.print_h5_keys(h5_file, return_list=True, do_print=False)
    with h5py.File(h5_file, 'r+') as h:
        if key in all_keys and overwrite_if_exists:
            print('key already exists, overwriting value...')
            del h[key]
            h.create_dataset(key, data=values)
        elif key in all_keys and not overwrite_if_exists:
            print("""key already exists, NOT overwriting value..., \nset 'overwrite_if_exists' to True to overwrite""")
        else:
            h.create_dataset(key, data=values)


for kk, h5 in zip(h_cont, all_h5s):
    F_NAME = os.path.basename(h5).split('Phil_')[-1][:-3]
    h52write = utils.lister_it(h5_list_to_write, keep_strings=F_NAME)[0]
    print(h52write)

    avg_cont = (np.mean(kk, axis=0) > .5) * 1
    tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 0]))
    for k in tmp1 + 1:
        avg_cont[k] = 0

    tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 1]))
    for k in tmp1 + 1:
        avg_cont[k] = 1

    tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 1, 0]))
    for k in tmp1 + 1:
        avg_cont[k] = 0
    for k in tmp1 + 2:
        avg_cont[k] = 0

    tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 0, 1]))
    for k in tmp1 + 1:
        avg_cont[k] = 1
    for k in tmp1 + 2:
        avg_cont[k] = 1

    tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 0]))
    assert tmp1.size == 0
    tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 1]))
    assert tmp1.size == 0
    tmp1 = search_sequence_numpy(avg_cont, np.asarray([0, 1, 1, 0]))
    assert tmp1.size == 0
    tmp1 = search_sequence_numpy(avg_cont, np.asarray([1, 0, 0, 1]))
    assert tmp1.size == 0

    utils.add_to_h5(h52write, 'labels', avg_cont, overwrite_if_exists=True)

utils.make_alt_labels_h5s(base_dir)
alt_label_dir = '/content/gdrive/My Drive/Colab data/curation_for_auto_curator/ALT_LABELS/'

to_pred_h5s = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED'
to_pred_h5s = utils.get_h5s(to_pred_h5s)

tmp1 = utils.get_h5s(alt_label_dir)
utils.print_h5_keys(tmp1[0])
for h5, h5_dest in zip(tmp1, to_pred_h5s):
    assert os.path.basename(h5) == os.path.basename(h5_dest)
    keys = utils.print_h5_keys(h5, return_list=True, do_print=False)
    for key in keys:
        utils.copy_h5_key_to_another_h5(h5, h5_dest, key)


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$  use color map to plot the predicitons with the images   $$$$$$$$$$$$$$$$$$$$$$$$"""

color_dict = dict()

cmap = cm.get_cmap('inferno')
color_list = [0, .2, .4, .5, .6, .8]
for i, k1 in enumerate(color_list):
  color_dict[i] = np.asarray(cmap(k1)[:-1])*255

# color_dict[0] =np.asarray(cmap(0)[:-1])*255 #(255, 230, 230)

# color_dict[2] = np.asarray(cmap(0.2)[:-1])*255#(0, 255, 153)
# color_dict[3] = np.asarray(cmap(0.4)[:-1])*255#(0, 255, 0)

# color_dict[1] = np.asarray(cmap(0.5)[:-1])*255#(51, 102, 0)

# color_dict[4] = np.asarray(cmap(0.6)[:-1])*255#(255, 153, 51)
# color_dict[5] = np.asarray(cmap(.8)[:-1])*255#(255, 51, 0)

img_width = 61
height = 20

tmp1, tmp2 = utils.group_consecutives(np.where(real_bool==1)[0])
border = 2
k +=1
pred_m = pred_m.astype(float)
inds = range(tmp1[k][0]-border, tmp1[k][-1]+1+border*2)
tmp1 = []

tmp1 = np.tile(np.repeat(pred_m[inds], img_width, axis=0), (height, 1))
tmp1 = np.vstack((tmp1, np.tile(np.repeat(real[inds], img_width, axis=0), (height, 1))))
tmp1 = np.vstack((tmp1, np.tile(np.repeat(pred_v[inds], img_width, axis=0), (height, 1))))


tmp1 = np.stack((tmp1,)*3, axis=-1)

for kk in np.unique(tmp1.astype(int)):
  tmp3 = np.where(tmp1 == kk)
  for i1, i2 in zip(tmp3[0], tmp3[1]):
    tmp1[i1, i2, :] = color_dict[kk]

tmp1 = tmp1.astype(int)
# tmp1 = np.round((tmp1/5)*255).astype(int)
# tmp1[:, :, 1] = .5
# tmp1[:, :, 0] = .5

plt.figure(figsize=(20, 10))
with h5py.File(actual_h5_img_file, 'r') as h:
  tmp2 = image_tools.img_unstacker(h['images'][inds[0]:inds[-1]+1], num_frames_wide=len(inds))
  tmp2 = np.vstack((tmp1, tmp2))
  plt.imshow(tmp2)



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$  use color map to plot the predicitons with the images  VVVV222222 $$$$$$$$$$$$$$$$$$$$$$$$"""



def foo2(in_list_of_arrays = [], touch_number = 0 , border = 4, height = 20, img_width = 61,
         color_list = [0, .5, .2, .3, .75, .85], cmap_col = 'inferno'):
  # in_list_of_arrays[0] needs to be the "true"" values
  if in_list_of_arrays == []:
    print('no input arrays, returning...')
    return
  color_dict = dict()
  cmap = cm.get_cmap(cmap_col)

  for i, k1 in enumerate(color_list):
    color_dict[i] = np.asarray(cmap(k1)[:-1])*255


  in_list_of_arrays = copy.deepcopy(in_list_of_arrays)
  tmp1, tmp2 = utils.group_consecutives(np.where(in_list_of_arrays[0]!=0)[0])

  inds = list(range(tmp1[touch_number][0]-border, tmp1[touch_number][-1]+1+border*2))

  tmp1 = np.tile(np.repeat(pred_m[inds], img_width, axis=0), (height, 1))
  for i, k in enumerate(in_list_of_arrays):
    k = k.astype(float)
    if i == 0:
      tmp1 = np.tile(np.repeat(k[inds], img_width, axis=0), (height, 1))
    else:
      tmp1 = np.vstack((tmp1, np.tile(np.repeat(k[inds], img_width, axis=0), (height, 1))))
  tmp1 = np.stack((tmp1,)*3, axis=-1)

  for kk in np.unique(tmp1.astype(int)):
    tmp3 = np.where(tmp1 == kk)
    for i1, i2 in zip(tmp3[0], tmp3[1]):
      tmp1[i1, i2, :] = color_dict[kk]

  tmp1 = tmp1.astype(int)
  with h5py.File(actual_h5_img_file, 'r') as h:
    tmp2 = image_tools.img_unstacker(h['images'][inds[0]:inds[-1]+1], num_frames_wide=len(inds))
    tmp2 = np.vstack((tmp1, tmp2))
    return tmp2

tmp2 = foo2(in_list_of_arrays = [real, pred_m, pred_v], touch_number = 201, border = 4, height = 20, img_width = 61,
        color_list = [0, .5, .2, .3, .85, .95], cmap_col = 'inferno')
plt.figure(figsize=(20, 10))
plt.imshow(tmp2)



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ get human agreed percentages $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

from whacc import utils
all_h5s = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/', print_h5_list=False)
h_cont, h_names = utils._get_human_contacts_(all_h5s)
print('--------\npercent fully agree\n--------')
for k in h_cont:
  a = np.mean(np.mean(k, axis=0)==1)
  b = np.mean(np.mean(k, axis=0)==0)
  print(np.round(a+b, 4))


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$ replace h5 labels with average predictions $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
from whacc import utils
import numpy as np

base_dir = '/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/'
h5_list_to_write = utils.get_h5s(base_dir)
all_h5s = utils.get_h5s('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/', print_h5_list=False)
h_cont, h_names = utils._get_human_contacts_(all_h5s)

for kk, h5 in zip(h_cont, all_h5s):
  F_NAME = os.path.basename(h5).split('Phil_')[-1][:-3]
  h52write = utils.lister_it(h5_list_to_write, keep_strings=F_NAME)[0]
  print(h52write)

  avg_cont = (np.mean(kk, axis = 0)>.5)*1
  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([0, 1, 0]))
  for k in tmp1+1:
    avg_cont[k] = 0

  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([1,0,1]))
  for k in tmp1+1:
    avg_cont[k] = 1

  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([0,1,1,0]))
  for k in tmp1+1:
    avg_cont[k] = 0
  for k in tmp1+2:
    avg_cont[k] = 0

  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([1,0,0,1]))
  for k in tmp1+1:
    avg_cont[k] = 1
  for k in tmp1+2:
    avg_cont[k] = 1

  # assert that these types of errors dont exist in the final version after correcting them above
  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([0, 1, 0]))
  assert tmp1.size == 0
  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([1,0,1]))
  assert tmp1.size == 0
  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([0,1,1,0]))
  assert tmp1.size == 0
  tmp1 = utils.search_sequence_numpy(avg_cont, np.asarray([1,0,0,1]))
  assert tmp1.size == 0

  utils.add_to_h5(h52write, 'labels', avg_cont, overwrite_if_exists=True)



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ Fully automate the generation of all types data folder using a single H5 file (normal with "color" channels) $$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$ WITH AUGMENTATION $$$$$$$$$ WITH AUGMENTATION $$$$$$$$$ WITH AUGMENTATION $$$$$$$$$ WITH AUGMENTATION $$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""

from whacc import utils
from whacc import image_tools
from keras.preprocessing.image import ImageDataGenerator
import h5py
import shutil
from imgaug import augmenters as iaa  # optional program to further augment data
subset_h5s = utils.get_h5s("/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/RETRAIN_H5_data/")


def foo_get_frame_nums_from_all_inds(h5_subset_file):
  tmp1 = image_tools.get_h5_key_and_concatenate([h5_subset_file], 'all_inds')
  tmp1, tmp2 = utils.group_consecutives(tmp1)
  frame_nums = []
  for k in tmp2:
    frame_nums.append(len(k))
  return frame_nums

base_dir_all = '/content/'
#**********$%$%$%$%$%%$
# these indicate areas where user may want to customize like make a test set for example
#**********$%$%$%$%$%%$

tmp_h5s = base_dir_all+'tmp_h5s/'
for h5_subset_file in tqdm(subset_h5s):
  a = h5_subset_file.split('/')[-1].split('RETRAIN_')[-1].split('.h5')[0]
  base_h5 =  base_dir_all + 'DATA/data_'+a
  single_frame_h5s = base_h5+'/single_frame/'

  Path(tmp_h5s).mkdir(parents = True, exist_ok = True)
  Path(single_frame_h5s).mkdir(parents = True, exist_ok = True)
  #vvvvv add the frame nums to the H5 file
  frame_nums = foo_get_frame_nums_from_all_inds(h5_subset_file)
  utils.add_to_h5(h5_subset_file, 'frame_nums', frame_nums, overwrite_if_exists = True)
  #^^^^^ add the frame nums to the H5 file
  #vvvvvv single frame and then split
  utils.reduce_to_single_frame_from_color(h5_subset_file, tmp_h5s+'temp.h5')

  split_h5s = image_tools.split_h5_loop_segments([tmp_h5s+'temp.h5'],
                                     split_percentages=[.8, .2], #**********$%$%$%$%$%%$
                                     temp_base_name= [single_frame_h5s+'train',single_frame_h5s+'val'], #**********$%$%$%$%$%%$
                                     chunk_size=10000,
                                     add_numbers_to_name=False,
                                     disable_TQDM=True,
                                     set_seed=0,
                                     color_channel=False)
  #^^^^^^ single frame and then split
  #vvvvvvvv convert to all types
  utils.make_all_H5_types(single_frame_h5s)  # auto generate all the h5 types using a single set of flat (no color) image H5
  utils.make_alt_labels_h5s(single_frame_h5s)  # auto generate the different types of labels
  #^^^^^^^^ convert to all types
  #vvvvv AUGMENT #**********$%$%$%$%$%%$
  datagen = ImageDataGenerator(rotation_range=360,  #
                                width_shift_range=.1,  #
                                height_shift_range=.1,  #
                                shear_range=.00,  #
                                zoom_range=.25,
                                brightness_range=[0.2, 1.2])  #
  gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
  num_aug = 1
  xxx = utils.get_h5s(base_h5)
  xxx = utils.lister_it(xxx, remove_string=['ALT_LABELS', 'single_frame'])
  for each_h5 in xxx:
    if 'images' in utils.print_h5_keys(each_h5, True, False):
      combine_list = [each_h5]

      shutil.rmtree(tmp_h5s)
      Path(tmp_h5s).mkdir(parents = True, exist_ok = True)
      for ii in range(2):
        new_H5_file = each_h5.split('.')[0] + '_AUG_' + str(ii) + '.h5' # create new file name based on teh original H5 name
        new_H5_file = tmp_h5s + os.path.basename(new_H5_file)
        combine_list.append(new_H5_file)
        h5creator = image_tools.h5_iterative_creator(new_H5_file, overwrite_if_file_exists=True,
                                                      close_and_open_on_each_iteration=True, color_channel=True)

        with h5py.File(each_h5, 'r') as hf:
          for image, label in zip(hf['images'][:], hf['labels'][:]):
            aug_img_stack, labels_stack = image_tools.augment_helper(datagen, num_aug, 0, image, label)
            aug_img_stack = gaussian_noise.augment_images(aug_img_stack) # optional
            h5creator.add_to_h5(aug_img_stack[:, :, :, :], labels_stack)
        utils.copy_h5_key_to_another_h5(each_h5, new_H5_file, 'frame_nums', 'frame_nums') # copy the frame nums to the sug files
      # combine all the
      image_tools.split_h5_loop_segments(combine_list,
                                         [1],
                                         each_h5.split('.h5')[0]+'_AUG.h5',
                                         add_numbers_to_name = False,
                                         set_seed=0,
                                         color_channel=True)


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ copy over the paths of the new set into one folder, the new version of above code should make this outdated $$"""

from distutils.dir_util import copy_tree
import glob

a = utils.lister_it(glob("/content/*"), keep_strings='/data_')
x = "xxxxxxxxxxxxx/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/ALL_RETRAIN_H5_data/"
shutil.rmtree(x)
Path(x).mkdir(parents = True, exist_ok = True)
for k in tqdm(a):
  Path(x).mkdir(parents = True, exist_ok = True)
  tmp1 = x+os.path.basename(k)
  Path(tmp1).mkdir(parents = True, exist_ok = True)
  copy_tree(str(k), tmp1)

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$print only the directories in a folder $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
# can use this to copy the folder structure if we want to
src = "/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/ALL_RETRAIN_H5_data"
for rootdir, dirs, files in os.walk(src):
    for subdir in dirs:
        print(os.path.join(src, subdir))






"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ Fully automate the generation of all types data folder using a single H5 file (normal with "color" channels) $$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
from whacc import utils
from whacc import image_tools
from keras.preprocessing.image import ImageDataGenerator
import h5py
import matplotlib.pyplot as plt
from imgaug import augmenters as iaa  # optional program to further augment data
#**********$%$%$%$%$%%$
# v^v^v^v^v^ these indacate areas where user may want to customize like make a test set for example
#**********$%$%$%$%$%%$
base_dir_all = '/content/'


full_session_h5s = "/content/gdrive/My Drive/Colab data/curation_for_auto_curator/H5_data/"
full_session_h5s = utils.get_h5s(full_session_h5s, 0)
b = utils.print_h5_keys(full_session_h5s[0], 1, 0)
c = utils.lister_it(b, remove_string='MODEL')
utils.print_list_with_inds(c)

for h5_subset_file in tqdm(full_session_h5s[:4]):
  a = h5_subset_file.split('/')[-1].split('RETRAIN_')[-1].split('.h5')[0]
  base_h5 =  base_dir_all + 'DATA_FULL/data_'+a
  single_frame_h5s = base_h5+'/single_frame/'

  # Path(tmp_h5s).mkdir(parents = True, exist_ok = True)
  Path(single_frame_h5s).mkdir(parents = True, exist_ok = True)

  #vvvvvv single frame
  single_name = single_frame_h5s+a+'.h5'
  utils.reduce_to_single_frame_from_color(h5_subset_file, single_name)
  #^^^^^^ single frame
  #vvvvvvvv convert to all types
  utils.make_all_H5_types(single_frame_h5s)  # auto generate all the h5 types using a single set of flat (no color) image H5
  utils.make_alt_labels_h5s(single_frame_h5s)  # auto generate the different types of labels
  #^^^^^^^^ convert to all types


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ transfer alt labels to the training and val set   from the ALT_LABELS source $$"""


x = "/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/ALL_RETRAIN_H5_data/"
for k in next(os.walk(x))[1]:
  a = utils.get_h5s(x+os.sep+k)
  a = utils.lister_it(a, keep_strings='train')
  src_h5 = utils.lister_it(a, '/ALT_LABELS/')[0]
  dest_h5s = utils.lister_it(a, remove_string='/ALT_LABELS/')
  for src_label_key in utils.print_h5_keys(src_h5, 1):
    for dest_H5 in dest_h5s:
      # print('from '+ src_h5 + ' to '+ dest_H5 + ' key--> '+ src_label_key)
      utils.copy_h5_key_to_another_h5(src_h5, dest_H5, src_label_key, src_label_key)

for k in next(os.walk(x))[1]:
  a = utils.get_h5s(x+os.sep+k)
  a = utils.lister_it(a, keep_strings='val')
  src_h5 = utils.lister_it(a, '/ALT_LABELS/')[0]
  dest_h5s = utils.lister_it(a, remove_string='/ALT_LABELS/')
  for src_label_key in utils.print_h5_keys(src_h5, 1):
    for dest_H5 in dest_h5s:
      # print('from '+ src_h5 + ' to '+ dest_H5 + ' key--> '+ src_label_key)
      utils.copy_h5_key_to_another_h5(src_h5, dest_H5, src_label_key, src_label_key)
# #testing it
# x = '/content/ALL_RETRAIN_H5_data/data_AH0698_170601_PM0121_AAAA/single_frame/train.h5'
# L1 = get_h5_key_and_concatenate(x, 'labels')
# L2 = get_h5_key_and_concatenate(x, '[0, 1]- (no touch, touch)')
# all(L1==L2)


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ transfer alt labels to the 'test' set  from the ALT_LABELS source$$"""


x = "/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/DATA_FULL/"

for k in next(os.walk(x))[1]:
  a = utils.get_h5s(x+os.sep+k)
  # a = utils.lister_it(a, keep_strings='train')
  src_h5 = utils.lister_it(a, '/ALT_LABELS/')[0]
  dest_h5s = utils.lister_it(a, remove_string='/ALT_LABELS/')
  for src_label_key in utils.print_h5_keys(src_h5, 1):
    for dest_H5 in dest_h5s:
      # print('from \n'+ src_h5 + ' to \n'+ dest_H5 + ' \nkey--> '+ src_label_key)
      utils.copy_h5_key_to_another_h5(src_h5, dest_H5, src_label_key, src_label_key)


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ copy over all of one h5 keys to another based on a matching string $$"""

model_3_h5s = "/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/h5_data_withMODEL3/"
model_3_h5s = utils.get_h5s(model_3_h5s)
print('\n')
to_pred_h5s = '/content/gdrive/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED'
to_pred_h5s = utils.get_h5s(to_pred_h5s)

for src, dest in tzip(model_3_h5s, to_pred_h5s):
  assert os.path.basename(src)[:15] == os.path.basename(dest)[:15]
  with h5py.File(src, 'r') as h5src:
    with h5py.File(dest, 'r+') as h5dest:
      for key in tqdm(h5src.keys()):
        if 'MODEL_3_' in key:
          try:
            h5dest.create_dataset(key, shape=np.shape(h5src[key][:]),data=h5src[key][:])
          except:
            del h5dest[key]
            time.sleep(4)
            h5dest.create_dataset(key, shape=np.shape(h5src[key][:]),data=h5src[key][:])



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ get legit pole times based on best information we haev it doesnt matter if there is an offset of the pole times
  so long as it is relative to the real pole triggers$$"""

def pole_relative_first_and_last_touches(h5_file, truth_contacts):
  try:
    frame_nums = image_tools.get_h5_key_and_concatenate([h5_file], 'frame_nums')
  except:
    tnfn = image_tools.get_h5_key_and_concatenate([h5_file], 'trial_nums_and_frame_nums')
    frame_nums = tnfn[1, :]

  pt = image_tools.get_h5_key_and_concatenate([h5_file], 'pole_times')
  first_touches = []
  last_touches = []
  for ii, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):

    x = truth_contacts[i1:i2]
    tmp1 = np.where(x)[0]
    if tmp1.size !=0:
      first_touches.append(tmp1[0]-pt[0, ii])
      last_touches.append(tmp1[-1]-pt[1, ii])
    else:
      first_touches.append(np.nan)
      last_touches.append(np.nan)
  min_first_touch = np.nanmin(first_touches)
  max_last_touch = np.nanmax(last_touches)

  return min_first_touch, max_last_touch, first_touches, last_touches

def define_in_range(h5_file, pole_up_add_to_trigger=0, pole_down_add_to_trigger=0, write_to_h5=True, return_in_range=False):
  try:
    fn = image_tools.get_h5_key_and_concatenate([h5_file], 'frame_nums')
  except:
    tnfn = image_tools.get_h5_key_and_concatenate([h5_file], 'trial_nums_and_frame_nums')
    fn = tnfn[1, :]
  with h5py.File(h5_file, 'r+') as hf:
    # print(hf['images'].shape)
    new_in_range = np.zeros(hf['images'].shape[0])
    # new_in_range = np.zeros_like(hf['in_range'][:]) # might have to add 'in_range'
    # # variable using length labels or something beforehand if it isnt done with video
    # # extraction automatically although I think it already does.
    for i, (i1, i2) in enumerate(utils.loop_segments(fn)):
      x = hf['pole_times'][:, i] + i1
      x1 = x[0] + pole_up_add_to_trigger
      x2 = x[1] + pole_down_add_to_trigger +1
      x2 = min([x2, i2])
      new_in_range[int(x1):int(x2)] = 1
    if write_to_h5:
      hf['in_range'][:] = new_in_range
    if return_in_range:
      return new_in_range




base_dir = '/content/ALT_LABELS_FINAL_PRED'
h5_list_to_write = utils.get_h5s(base_dir, 0)
for k in h5_list_to_write:

  # avg_human_cont = image_tools.get_h5_key_and_concatenate([k], 'labels')
  truth_contacts = image_tools.get_h5_key_and_concatenate([k], '[0, 1]- (no touch, touch)')
  min_first_touch, max_last_touch, first_touches, last_touches = pole_relative_first_and_last_touches(k, truth_contacts)
  in_range = define_in_range(k,
                              pole_up_add_to_trigger=min_first_touch,
                              pole_down_add_to_trigger=max_last_touch,
                              write_to_h5=True,
                              return_in_range=True)
  print(min_first_touch)
  print(max_last_touch)
  print(np.sum(abs(in_range-1)*truth_contacts))
  print('\n')
"""
some extra code for copying all the info here to a list of them 




"""

x = "/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/DATA_FULL"
a = utils.get_h5s(x)
TO_GET_IN_RANGE_H5S = utils.lister_it(a, remove_string='/ALT_LABELS/')

for k in tqdm(TO_GET_IN_RANGE_H5S):

  # avg_human_cont = image_tools.get_h5_key_and_concatenate([k], 'labels')
  truth_contacts = image_tools.get_h5_key_and_concatenate([k], '[0, 1]- (no touch, touch)')
  min_first_touch, max_last_touch, first_touches, last_touches = pole_relative_first_and_last_touches(k, truth_contacts)
  in_range = define_in_range(k,
                              pole_up_add_to_trigger=min_first_touch,
                              pole_down_add_to_trigger=max_last_touch,
                              write_to_h5=False,
                              return_in_range=True)
  first_touch = min_first_touch
  last_touch = max_last_touch

  with h5py.File(k, 'r+') as hf:
    tmp1 = ['in_range', 'first_touch', 'first_touches', 'last_touch', 'last_touches']
    for key_name in tmp1:
      data = eval(key_name)
      try:
        hf.create_dataset(key_name,data=data)
      except:
        del hf[key_name]
        time.sleep(1)
        hf.create_dataset(key_name,data=data)

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ rename the model predictions on the regular_80_FINAL_PRED set"""

# rename the model predictions on the regular_80_FINAL_PRED set
new_names = utils.lister_it(utils.print_h5_keys(to_pred_h5s[0], 1, 0), keep_strings='MODEL_', remove_string='viterbi')

for h5_file in all_pred_h5:
  with h5py.File(h5_file, 'r+') as hf:
    for k in new_names:
      old_names = utils.lister_it(utils.print_h5_keys(h5_file, 1, 0), keep_strings='MODEL_')
      k2 = k.split('_-_')[0]
      x = np.where(k2 == np.asarray(old_names))[0]
      if len(x)!=1:
        print(x)
      else:
        x = x[0]
        hf.create_dataset(k, data = hf[old_names[x]][:])
        del hf[old_names[x]]


"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ crop out pole down times to get in range times in_range"""


all_h5s = utils.get_h5s("/content/DATA_FULL/", 0)
all_h5s = utils.lister_it(all_h5s, remove_string='/ALT_LABELS/')

all_in_range_h5s = utils.get_h5s('/content/gdrive/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED')


keys_to_crop = ['[0, 1, 2, 3, 4, 5]- (no touch, touch, onset, one after onset, offset, one after offset)',
 '[0, 1, 2, 3]- (no touch, touch, one after onset, offset)',
 '[0, 1, 2, 3]- (no touch, touch, onset, offset',
 '[0, 1, 2]- (no event, onset, offset)',
 '[0, 1]- (no touch, touch)',
 '[0, 1]- (not offset, offset)',
 '[0, 1]- (not onset, onset)',
 'labels',
 'images']
keys_to_keep = ['multiplier',
 'frame_nums',
 'file_name_nums',
 'pole_times'] + keys_to_crop


for h5 in tqdm(all_h5s):
  base_name = h5.split('data_')[-1].split('/')[0]
  in_range_h5 = utils.lister_it(all_in_range_h5s, keep_strings=[base_name])
  assert len(in_range_h5) == 1
  in_range = image_tools.get_h5_key_and_concatenate(in_range_h5[0], 'in_range')

  original_total = len(in_range)
  with h5py.File(h5, 'r+') as hf:
    for tmp_key in hf.keys():#first delete all irrelivent keys
      if tmp_key not in keys_to_keep:
        del hf[tmp_key]
    for key in keys_to_crop:
      new_frame_nums = []#hacky but fine
      cropped_var = None
      for i1, i2 in utils.loop_segments(hf['frame_nums'][:]):
        ir = in_range[i1:i2]
        new_frame_nums.append(np.sum(ir))

        tmp1 = hf[key][i1:i2]
        tmp2 = tmp1[ir.astype(bool)]
        if cropped_var is None:
          cropped_var = tmp2
        elif len(tmp1.shape) == 1:
          cropped_var = np.concatenate((cropped_var, tmp2))
        else:
          cropped_var = np.vstack((cropped_var, tmp2))
      del hf[key]
      hf.create_dataset(key, data = cropped_var)
    del hf['frame_nums']
    hf.create_dataset('frame_nums', data = new_frame_nums)
    hf.create_dataset('original_total', data = original_total)


# download augmented and transformed subsets
target_path = "/content/gdrive/MyDrive/Colab data/curation_for_auto_curator/DATA_FULL_in_range_only/"
source_path = "/content/DATA_FULL"
Path(target_path).mkdir(parents = True, exist_ok = True)

sync(source_path, target_path, 'sync')



"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ cute the model head after global pooling and just use the features on a new model like LGM or SVM """


def convert_h5_to_feature_h5(model,in_generator, h5_new_full_file_name):
  h5c = image_tools.h5_iterative_creator(h5_new_full_file_name,
                                        overwrite_if_file_exists=True,
                                        max_img_height=1,
                                        max_img_width=2048,
                                        close_and_open_on_each_iteration=True,
                                        color_channel=False,
                                        add_to_existing_H5=False,
                                        ignore_image_range_warning=False,
                                        dtype_img = h5py.h5t.IEEE_F32LE,
                                        dtype_labels = h5py.h5t.IEEE_F32LE)

  for k in tqdm(range(in_generator.__len__())):
    x, y = in_generator.__getitem__(k)
    features = model_out.predict(x)
    h5c.add_to_h5(features, y)
for in_gen in [test_gen, val_gen, train_gen]:
  new_name = '/feature_data/'.join(in_gen.H5_file_list[0].split('/data/'))
  new_name = ''.join(new_name.split('____temp'))
  Path(os.path.dirname(new_name)).mkdir(parents=True, exist_ok=True)
  convert_h5_to_feature_h5(model,in_gen, new_name)

"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"""
"""$$ copy alt labels to other files  """



def find_alt_labels_file(h5_in):
  alt_label_path = os.sep.join(os.path.dirname(h5_in).split(os.sep)[:-1])+os.sep+'ALT_LABELS'
  if os.path.isdir(alt_label_path):
    labels_file = utils.get_h5s(alt_label_path)
    if len(labels_file) == 1:
      return labels_file[0]
    else:
      print('more than one file found in directory, ambiguous, returning None for file ...')
      print(h5_in)
      return None
  else:
    print('Valid alt labels path not found, returning None for file ...')
    print(h5_in)
    return None

ind = 0
labels_file = find_alt_labels_file(src_files[ind])

utils.copy_over_all_non_image_keys(labels_file, src_files[ind])



from whacc import image_tools,model_maker, feature_maker

h5_in = '/Users/phil/Desktop/pipeline_test/AH0407x160609_3lag.h5'
h5_FD = h5_in.replace('.h5', '_feature_data.h5')

mod = model_maker.load_final_model()
in_gen = image_tools.ImageBatchGenerator(500, h5_in, label_key='labels')
feature_maker.convert_h5_to_feature_h5(mod, in_gen, h5_FD)



import os

from whacc import utils, image_tools
import numpy as np
from tqdm.autonotebook import tqdm

bd_2105 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/FEATURE_DATA/colab_data2/model_testing/all_data/all_models_2105/regular_80_border/data/3lag/'
tmp1 = utils.get_h5s(bd_2105)
train_1_2105 = tmp1[0]
val_1_2105 = tmp1[1]

bd_labels = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/all_models/regular_80_border/data/ALT_LABELS/'
tmp1 = utils.get_h5s(bd_labels)
train_1_labels = tmp1[0]
val_1_labels = tmp1[1]

bd_full_h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/all_models/regular_80_border/data/3lag/'
tmp1 = utils.get_h5s(bd_full_h5)
train_1_fn = tmp1[0]
val_1_fn = tmp1[1]

data_dict = dict()
data_dict['train_1'] = dict()#{'data', 'frame_nums', 'labels'}
data_dict['val_1'] = dict()#{'data', 'frame_nums', 'labels'}
data_dict['test_2'] = dict()#{'data', 'frame_nums', 'labels'}


data_dict['train_1']['data'] = image_tools.get_h5_key_and_concatenate(train_1_2105, 'final_features_2105')
data_dict['val_1']['data'] = image_tools.get_h5_key_and_concatenate(val_1_2105, 'final_features_2105')

data_dict['train_1']['labels'] = image_tools.get_h5_key_and_concatenate(train_1_labels, '[0, 1]- (no touch, touch)')
data_dict['val_1']['labels'] = image_tools.get_h5_key_and_concatenate(val_1_labels, '[0, 1]- (no touch, touch)')

data_dict['train_1']['frame_nums'] = image_tools.get_h5_key_and_concatenate(train_1_fn, 'frame_nums')
data_dict['val_1']['frame_nums'] = image_tools.get_h5_key_and_concatenate(val_1_fn, 'frame_nums')


tmp1 = [data_dict['train_1']['data'].shape[0], data_dict['train_1']['labels'].shape[0], np.sum(data_dict['train_1']['frame_nums'])]
assert np.all(tmp1[0]==np.asarray(tmp1)), 'dataset doesnt match labels and/or frame nums'

tmp1 = [data_dict['val_1']['data'].shape[0], data_dict['val_1']['labels'].shape[0], np.sum(data_dict['val_1']['frame_nums'])]
assert np.all(tmp1[0]==np.asarray(tmp1)), 'dataset doesnt match labels and/or frame nums'

"""
LOAD DATA 
"""
# bd_fn = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/frame_nums_for_cnn_comparison'
# bd_data = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/final_2105_for_cnn_comparison'
# bd_labels = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/labels_for_cnn_comparison'

replace_data = r'final_2105_for_cnn_comparison'
replace_labels = r'labels_for_cnn_comparison'
replace_frame_nums = r'frame_nums_for_cnn_comparison'

bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/'
bd_data = bd + replace_data

data_files = utils.sort(utils.get_files(bd_data, '*.npy'))

tvt_x = [[],[],[]]
tvt_y = [[],[],[]]
tvt_fn = [[],[],[]]

for set_i, set_type in enumerate(['test_']):
    data_files_set = utils.sort(utils.lister_it(data_files, set_type))

    for f_data in tqdm(data_files_set):
        f_label = f_data.replace(replace_data, replace_labels)
        f_fn = f_data.replace(replace_data, replace_frame_nums)

        tvt_x[set_i].append(np.load(f_data, allow_pickle=True))
        tvt_y[set_i].append(np.load(f_label, allow_pickle=True))
        tvt_fn[set_i].append(np.load(f_fn, allow_pickle=True))

    data_dict['test_2']['data'] = np.concatenate(tvt_x[set_i])
    data_dict['test_2']['labels'] = np.concatenate(tvt_y[set_i])
    data_dict['test_2']['frame_nums'] = np.concatenate(tvt_fn[set_i])
del tvt_x, tvt_y, tvt_fn

tmp1 = [data_dict['test_2']['data'].shape[0], data_dict['test_2']['labels'].shape[0], np.sum(data_dict['test_2']['frame_nums'])]
assert np.all(tmp1[0]==np.asarray(tmp1)), 'dataset doesnt match labels and/or frame nums'


bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/'
utils.save_obj(data_dict, bd + os.sep + 'T1V1t2_data_dict')



tmp_dict = utils.load_obj(bd+os.sep+'T1V1t2_data_dict.pkl')

tvt_x = [[],[],[]]
tvt_y = [[],[],[]]
tvt_fn = [[],[],[]]

tvt_x[0] = tmp_dict['train_1']['data']
tvt_x[1] = tmp_dict['val1']['data']
tvt_x[2] = tmp_dict['test_2']['data']

tvt_y[0] = tmp_dict['train_1']['frame_nums']
tvt_y[1] = tmp_dict['val1']['frame_nums']
tvt_y[2] = tmp_dict['test_2']['frame_nums']

tvt_fn[0] = tmp_dict['train_1']['labels']
tvt_fn[1] = tmp_dict['val1']['labels']
tvt_fn[2] = tmp_dict['test_2']['labels']

del tmp_dict




from whacc import utils
from whacc import image_tools
from keras.preprocessing.image import ImageDataGenerator
import h5py
import matplotlib.pyplot as plt
from tqdm import tqdm

from imgaug import augmenters as iaa  # optional program to further augment data
'*remove_2_edges.h5'
h5_subset_file = '/Users/phil/Dropbox/Autocurator/data/samsons_subsets/use/train_and_validate/validation_set.h5'
# datagen = ImageDataGenerator(rotation_range=360,  #
#                              width_shift_range=.05,  #
#                              height_shift_range=.05,  #
#                              shear_range=.05,  #
#                              zoom_range=.1,
#                              brightness_range=[0.2, 1.1])  #

datagen = ImageDataGenerator(rotation_range=360,  #
                             width_shift_range=.05,  #
                             height_shift_range=.05,  #
                             shear_range=.00,  #
                             zoom_range=.02,
                             brightness_range=[0.2, 1.1])  #

with h5py.File(h5_subset_file, 'r') as hf:
    test_img = hf['images'][0]  # grab a single image
num_aug = 4

aug_ims, _ = image_tools.augment_helper(datagen, num_aug, 0 ,test_img, -1)  # make 99 of the augmented images and output 1 of th original
# '-1' is the label associate ith the image, we dont care becasue we just want the iamge so it doenst matter what it is here

plt.figure(figsize=[5, 5])
plt.imshow(image_tools.img_unstacker(aug_ims, 10), )
plt.show()

gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
aug_ims_2 = gaussian_noise.augment_images(aug_ims)
plt.figure(figsize=[10, 10])
plt.imshow(image_tools.img_unstacker(aug_ims_2, 10))
plt.show()



# once we are happy with our augmentation process we can make an augmented H5 file using class
new_H5_file = h5_subset_file.split('.')[0] + '_AUG_min.h5' # create new file name based on teh original H5 name
h5creator = image_tools.h5_iterative_creator(new_H5_file, overwrite_if_file_exists=True,
                                             close_and_open_on_each_iteration=True)


utils.get_class_info(h5creator)
with h5py.File(h5_subset_file, 'r') as hf:
    for image, label in zip(tqdm(hf['images'][:]), hf['labels'][:]):
        aug_img_stack, labels_stack = image_tools.augment_helper(datagen, num_aug, 0, image, label)
        aug_img_stack = gaussian_noise.augment_images(aug_img_stack) # optional
        h5creator.add_to_h5(aug_img_stack, labels_stack)



#  test augment pattern on a blank pattern
# import numpy as np
# gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
# aug_ims_2 = gaussian_noise.augment_image(100*np.ones_like(aug_ims[0])).astype('int16')
# aug_ims_2[0, 0, :] = 255
# aug_ims_2[0, 1, :] = 0
# plt.figure(figsize=[5, 5])
# plt.imshow(aug_ims_2)
# plt.show()
# print(np.unique(aug_ims_2))


# #-=-=-=-=--=-=-=-=-=-=-
# from whacc import utils
# from whacc import image_tools
# from keras.preprocessing.image import ImageDataGenerator
# import h5py
# import matplotlib.pyplot as plt
# from tqdm import tqdm
#
# from imgaug import augmenters as iaa  # optional program to further augment data
#
# h5_subset_file = '/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/ALL_HUM_CONT_161.h5'
# num_aug = 4
# xxx = '_AUG_more_zoom_and_shift.h5'
#
# datagen = ImageDataGenerator(rotation_range=360,  #
#                              width_shift_range=.1,  #
#                              height_shift_range=.1,  #
#                              shear_range=.00,  #
#                              zoom_range=.1,
#                              brightness_range=[0.2, 1.1])  #
#
# with h5py.File(h5_subset_file, 'r') as hf:
#     test_img = hf['images'][85]  # grab a single image
#
# aug_ims, _ = image_tools.augment_helper(datagen, num_aug, 0 ,test_img, -1)  # make 99 of the augmented images and output 1 of th original
# # '-1' is the label associate ith the image, we dont care becasue we just want the iamge so it doenst matter what it is here
#
# plt.figure(figsize=[5, 5])
# plt.imshow(image_tools.img_unstacker(aug_ims, 10), )
# plt.show()
#
# gaussian_noise = iaa.AdditiveGaussianNoise(loc = 0, scale=3)
# aug_ims_2 = gaussian_noise.augment_images(aug_ims)
# plt.figure(figsize=[10, 10])
# plt.imshow(image_tools.img_unstacker(aug_ims_2, 10))
# plt.show()
#
#
#
# # once we are happy with out augmentation process we can make an augmented H5 file using class
# new_H5_file = h5_subset_file.split('.')[0] + xxx # create new file name based on teh original H5 name
# h5creator = image_tools.h5_iterative_creator(new_H5_file, overwrite_if_file_exists=True,
#                                              close_and_open_on_each_iteration=True,
#                                              color_channel=False)
#
#
# utils.get_class_info(h5creator)
# with h5py.File(h5_subset_file, 'r') as hf:
#     for image, label in zip(tqdm(hf['images'][:]), hf['labels'][:]):
#         aug_img_stack, labels_stack = image_tools.augment_helper(datagen, num_aug, 0, image, label)
#         aug_img_stack = gaussian_noise.augment_images(aug_img_stack) # optional
#         h5creator.add_to_h5(aug_img_stack[:, :, :, 0], labels_stack)







from whacc import utils
from whacc import image_tools

m_list = utils.get_model_list('/Users/phil/Dropbox/Colab data/model_iterations')
model_2_load = m_list[12]
# model_2_load = m_list[6]


H5_list = utils.get_h5s('/Users/phil/Dropbox/Autocurator/testing_data/MP4s/')
H5_file = image_tools.predict_multiple_H5_files(H5_list,
                                                model_2_load,
                                                append_model_and_labels_to_name_string = False,
                                                batch_size = 1000,
                                                save_on = True,
                                                label_save_name = 'labels')

# import h5py
# import matplotlib.pyplot as plt
# for k in H5_list:
#     with h5py.File(k, 'r') as hf:
#             plt.plot(hf['labels'][:])



# import h5py
# import matplotlib.pyplot as plt
# h5_file_name = '/Users/phil/Dropbox/Autocurator/testing_data/MP4s/AH0667x170317_JON/AH0667x170317-.h5'
# with h5py.File(h5_file_name, 'r') as hf:
#     # print(hf['labels'])
#     for k in hf.keys():
#         print(k)
#     plt.plot(hf['MODEL__resnet50 regular images full unfrozen20201028FULL__labels'][:])
# #



# from google.colab import drive
# # drive.mount('/content/gdrive')
# drive._mount('/content/gdrive')
import shap

import pdb
import lightgbm as lgb
from datetime import datetime, timedelta

from whacc import model_maker

from whacc.model_maker import *


import h5py

from tqdm.autonotebook import tqdm


import pickle
from whacc import utils, image_tools, transfer_learning, analysis
import matplotlib.pyplot as plt

from sklearn.utils import class_weight
import os
import copy
import numpy as np

from pathlib import Path
import shutil
import zipfile
import pytz
import json
from math import isclose, sqrt
from IPython import display


import pandas as pd
import time


import seaborn as sns
from matplotlib.collections import PathCollection
import itertools

from natsort import os_sorted
import joblib
import seaborn as sns

import multiprocessing
# from natsort import os_sorted
from natsort import natsorted, ns
## get feature names



def foo_plot2():
    from matplotlib import cm

    early_stopping_num = 500
    x = np.diff(1-d['auc'])
    peaks = np.where(x>0.001)[0]+1
    loop_peaks = np.diff([0]+list(peaks)+[len(d['auc'])])
    fig, ax1 = plt.subplots(figsize=[20, 5])
    ax2 = ax1.twinx()
    tot = len(loop_peaks)

    inds = np.linspace(.15, .95, len(loop_peaks))
    Blues = cm.get_cmap('Blues')([inds])
    Reds = cm.get_cmap('Reds')([inds])

    for i, (k1, k2) in enumerate(utils.loop_segments(loop_peaks)):
        c1 = i+1/tot

        d2 = {'auc':d['auc'][k1:k2], 'touch_count_errors_per_touch':d['touch_count_errors_per_touch'][k1:k2]}
        color = Blues[0][i]
        ax1.set_ylabel('1-auc', color=color)  # we already handled the x-label with ax1
        ax1.plot(1-d2['auc'], color=color)
        ax1.tick_params(axis='y', labelcolor=color)


        color = Reds[0][i]
        ax2.set_xlabel('touch_count_errors_per_touch')
        ax2.set_ylabel('touch_count_errors_per_touch', color=color)
        ax2.plot(d2['touch_count_errors_per_touch'], color=color)
        ax2.tick_params(axis='y', labelcolor=color)

        L = len(d2['auc'])

        b = np.argmin(np.flip(1-d2['auc'][-early_stopping_num:]))
        b = np.flip(np.arange(early_stopping_num))[b]
        a = b+L-early_stopping_num
        ax1.plot(a, 1-d2['auc'][a], 'g.')

        b = np.argmin(np.flip(d2['touch_count_errors_per_touch'][-early_stopping_num:]))
        b = np.flip(np.arange(early_stopping_num))[b]
        a = b+L-early_stopping_num
        ax2.plot(a, d2['touch_count_errors_per_touch'][a], 'g.')
    ax1.set_ylim([0.00055, .0008])
    ax2.set_ylim([.05, .065])


    ax1.set_ylim([0.00, .00075])
    ax2.set_ylim([.05, .095])

#     ax1.set_ylim([0.00, 1-.98])
#     ax2.set_ylim([.05, .3])
    plt.grid()
    return ax1, ax2

def foo_plot():
    early_stopping_num = 500

    fig, ax1 = plt.subplots(figsize=[20, 5])
    color = 'tab:blue'
    ax1.set_ylabel('1-auc', color=color)  # we already handled the x-label with ax1
    ax1.plot(1-d['auc'], color=color)
    ax1.tick_params(axis='y', labelcolor=color)
    plt.ylim([0.00055, .0008])
    # plt.ylim([0.00055, .0006])
    # plt.ylim([0.000, .002])
    # plt.ylim([0.000, .008])
    ax2 = ax1.twinx()
    color = 'tab:red'
    ax2.set_xlabel('touch_count_errors_per_touch')
    ax2.set_ylabel('touch_count_errors_per_touch', color=color)
    ax2.plot(d['touch_count_errors_per_touch'], color=color)
    ax2.tick_params(axis='y', labelcolor=color)
    # plt.ylim([.028, .034])
    # plt.ylim([.028, .044])
    plt.ylim([.05, .065])
    # plt.ylim([.04, .14])
    # plt.ylim([.056, .0571])
    L = len(d['auc'])

    b = np.argmin(np.flip(1-d['auc'][-early_stopping_num:]))
    b = np.flip(np.arange(early_stopping_num))[b]
    a = b+L-early_stopping_num
    ax1.plot(a, 1-d['auc'][a], 'g.')

    b = np.argmin(np.flip(d['touch_count_errors_per_touch'][-early_stopping_num:]))
    b = np.flip(np.arange(early_stopping_num))[b]
    a = b+L-early_stopping_num
    ax2.plot(a, d['touch_count_errors_per_touch'][a], 'g.')

    plt.grid()
    # plt.xlim([L-early_stopping_num+1, L+5])
    # plt.xlim([L-401, L+5])
    # plt.xlim([L-1001, L+5])

    # plt.xlim([3500, 4300])


    plt.plot()

    ax2.axhline(np.min(d['touch_count_errors_per_touch']), color='k')
    print(np.min(d['touch_count_errors_per_touch']))
    plt.figure()
    plt.plot(d['best_threshold'])
    plt.ylim([.1, 1])
    plt.grid()
    """
    one thing to test is if optuna is saving the best AUC or if it is following the early stopping from the light GBM 
    """

#
# bd = r'Q:\WhACC_DATA_FINAL_WITH_AUG\LGBM_models_for_cnn_comparison_V1\\'
# bd = r'Q:\WhACC_DATA_FINAL_WITH_AUG\LGBM_models_for_cnn_comparison_early_stop_500_V2\\'
# bd = r'Q:\WhACC_DATA_FINAL_WITH_AUG\LGBM_models_for_cnn_comparison_T1V1t2_V3\\'

bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/models_2/LGBM_models_for_cnn_comparison_T1V1t2_V3'

results_list = []
for k in utils.sort(utils.get_files(bd, '*_model_results.pkl')):
    results_list.append(utils.load_obj(k))


val_auc = np.asarray([k['metrics']['auc'] for k in results_list])
test_auc = np.asarray([k['metrics_test']['auc'] for k in results_list])
val_TCE = np.asarray([k['metrics']['touch_count_errors_per_touch'] for k in results_list])
test_TCE = np.asarray([k['metrics_test']['touch_count_errors_per_touch'] for k in results_list])

val_auc = np.asarray([k['metrics']['auc'] for k in results_list])
test_auc = np.asarray([k['metrics_test']['auc'] for k in results_list])
val_TCE = np.asarray([k['metrics']['touch_count_errors_per_touch'] for k in results_list])
test_TCE = np.asarray([k['metrics_test']['touch_count_errors_per_touch'] for k in results_list])


val_TCE_min = np.asarray([min(k) for k in val_TCE])
inds = np.where(val_TCE_min<.04)[0]
# inds = [66]
# inds = [8]
val_auc_min = np.asarray([min(1-np.asarray(k)) for k in val_auc])
inds = np.where(val_auc_min<.000769)[0]
inds = [78]

d = dict()
d['auc'] = np.concatenate(val_auc[inds])
d['touch_count_errors_per_touch'] = np.concatenate(val_TCE[inds])
ax1, ax2 = foo_plot2()
ax1.set_ylim([0.00, 0.002])
ax1.set_ylim([0.0006, 0.001])
ax2.set_ylim([.025, .06])


d = dict()
d['auc'] = np.concatenate(test_auc[inds])
d['touch_count_errors_per_touch'] = np.concatenate(test_TCE[inds])
ax1, ax2 = foo_plot2()
ax1.set_ylim([0.005, 0.015])
ax2.set_ylim([.14, .22])

"""
inds = [78]
This is ind 78 and the best model based on AUC for val data, 
so this is what I will choose to be fair since the other (CNN) 
models were chosen based on this same criteria. 

"""



# import numpy as np
# from whacc import utils
#
#
# class error_analysis():
#     def __init__(self, real_bool, pred_bool, frame_num_array=None):
#         self.real = real_bool
#         self.pred = pred_bool
#         self.type_list = ['ghost', 'ghost', 'append', 'miss', 'miss', 'deduct', 'join', 'split']
#         self.group_inds_neg = []
#         self.group_inds_pos = []
#         self.error_neg = []
#         self.error_pos = []
#         if frame_num_array is None:
#             frame_num_array = [len(self.pred)]
#
#         for i, (i1, i2) in enumerate(self.loop_segments(frame_num_array)):  # separate the trials
#             d = self.get_diff_array(self.real[i1:i2], self.pred[i1:i2])
#
#             R_neg, P_neg, X_neg, group_inds_neg = self.get_error_segments_plus_border(d, -1)
#             self.group_inds_neg += group_inds_neg
#             for each_x in X_neg:
#                 self.error_neg.append(self.get_error_type(each_x))
#
#             R_pos, P_pos, X_pos, group_inds_pos = self.get_error_segments_plus_border(d, 1)
#             self.group_inds_pos += group_inds_pos
#             for each_x in X_pos:
#                 self.error_pos.append(self.get_error_type(each_x))
#
#     @staticmethod
#     def loop_segments(frame_num_array):
#         frame_num_array = [0] + frame_num_array
#         frame_num_array = np.cumsum(frame_num_array)
#         return zip(list(frame_num_array[:-1]), list(frame_num_array[1:]))
#     @staticmethod
#     def one_sided_get_type(x):
#         if np.isnan(x[0]) and np.isnan(x[-1]):
#             if x[1] == -1:  # full trial ghost touch
#                 return -7
#             elif x[1] == 1:  # full trial miss touch
#                 return -4
#         if np.isnan(x[0]):  # edge case (error being parsed is @ index 0 or -1)
#             return -1
#         #                                          ghost.   ghost.   append.  miss.   miss.    deduct.
#         ind = np.where(np.all(x[:2] == np.asarray([[0, -1], [1, -1], [2, -1], [0, 1], [-1, 1], [2, 1]]), axis=1))[0][0]
#         return ind
#     @staticmethod
#     def type_parser(x):
#         minx = np.min(x)
#         maxx = np.max(x)
#         sort_vals = [minx, maxx]
#         if np.any(np.all(sort_vals == np.asarray([[1, 1], [1, 2], [2, 2]]), axis=1)):  # joins
#             return 6
#         elif maxx == minx and maxx == 5:  # splits
#             return 7
#         else:
#             return maxx  # everything else is already the correct index return the max
#
#     def get_error_type(self, x):
#         return self.type_parser([self.one_sided_get_type(x), self.one_sided_get_type(np.flip(x))])
#     @staticmethod
#     def get_diff_array(real_bool, pred_bool):
#         diff_array = np.asarray(real_bool) - np.asarray(pred_bool)
#         diff_array = diff_array + (real_bool + pred_bool == 2) * 2  # TP = 2, TN = 0, FP = -1, FN = 1
#         return diff_array
#     @staticmethod
#     def get_error_segments_plus_border(d, error_num_one_or_neg_one):
#         group_inds, _ = utils.group_consecutives(np.where(d == error_num_one_or_neg_one)[0])
#         R = []
#         P = []
#         X = []
#         for k in group_inds:
#             R_ind = k[-1] + 2
#             L_tmp = k[0] - 1
#             L_ind = np.max([L_tmp, 0])
#             #  for this error segment (plus edges) for ...
#             r = real_bool[L_ind:R_ind]  # real,
#             p = pred_bool[L_ind:R_ind]  # predicted and
#             x = d[L_ind:R_ind]  # the diff array (all match types).
#
#             if L_tmp < 0:  # edge case to the left
#                 r = np.append(np.nan, r)
#                 p = np.append(np.nan, p)
#                 x = np.append(np.nan, x)
#             if R_ind > len(d):  # edge case to the right
#                 r = np.append(r, np.nan)
#                 p = np.append(p, np.nan)
#                 x = np.append(x, np.nan)
#             R.append(r)
#             P.append(p)
#             X.append(x)
#
#         return R, P, X, group_inds
#
#
#
# from whacc import analysis
# import numpy as np
# real_bool = np.asarray([0, 0, 1, 1, 1, 1, 1])
# pred_bool = np.asarray([0, 1, 1, 0, 0, 0, 0])
# a = analysis.error_analysis(real_bool, pred_bool,[3,4])

from whacc import transfer_learning as tl
from whacc import image_tools
from whacc import utils

H5_list_to_train = ['/Users/phil/Dropbox/Colab data/RETRAIN_H5_data/RETRAIN_AH0407_160613_JC1003_AAAC.h5']
testing_h5_list = ['/Users/phil/Dropbox/Colab data/H5_data/AH0407_160613_JC1003_AAAC-006.h5']
testing_h5_list = H5_list_to_train


bd = '/Users/phil/Downloads/'
split_h5_files = image_tools.split_h5(H5_list_to_train, [8, 3], temp_base_name=[bd + 'training_set', bd + 'validation_set'],
                                      add_numbers_to_name=False)

validation_h5_list = [split_h5_files[1]]
training_h5_list = [split_h5_files[0]]

# all_models = utils.get_model_list("/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/model_iterations/model_iterations")
all_models = utils.get_model_list("/Users/phil/Dropbox/Colab data/model_iterations/")

batch_size = 50

# import tensorflow as tf
# model2 = tf.keras.models.load_model(all_models[12])

save_and_plot_history_1 = tl.save_and_plot_history(image_tools.ImageBatchGenerator(batch_size, testing_h5_list),
                                                   key_name='labels', plot_metric_inds=[1, 2])

training_info = tl.foo_start_running(all_models[12], training_h5_list, validation_h5_list, dropout=None, epochs=1000,
                                     batch_size=batch_size, learning_rate=10 ** -4, patience=10, monitor='val_loss',
                                     key_name='labels', verbose_=1, save_name=None, model_save_dir=None,
                                     save_best_only=False, add_callback_list=[save_and_plot_history_1])






import matplotlib.pyplot as plt
from tqdm.auto import tqdm
from imgaug import augmenters as iaa  # optional program to further augment data

from whacc import utils
import numpy as np
from whacc import image_tools, PoleTracking
from natsort import natsorted, ns
import pickle
import pandas as pd
import os
import copy
import seaborn as sns
from keras.preprocessing.image import ImageDataGenerator
import h5py

from whacc import utils, analysis
import h5py
import matplotlib.pyplot as plt
import numpy as np
from sklearn import metrics
from pathlib import Path

from whacc import utils, image_tools, transfer_learning, analysis
from IPython.utils import io
import matplotlib.pyplot as plt
from tensorflow.keras.layers import Dense, Dropout
from tensorflow.keras.models import Model
from tensorflow import keras
from sklearn.utils import class_weight
import time
from pathlib import Path
import os
import copy
import numpy as np
from tensorflow.keras import applications
from pathlib import Path
import shutil
import zipfile
from datetime import datetime
import pytz
import json
from whacc import model_maker

from whacc.model_maker import *
import itertools

import tensorflow as tf
from tensorflow import keras
import matplotlib.pyplot as plt
import numpy as np
import h5py
from whacc import image_tools
from whacc import utils
import copy
import time
import os
import pdb
import glob
from tqdm.contrib import tzip
import scipy.io as spio
import h5py
# from tqdm.notebook import tqdm
from matplotlib import cm

import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import numpy as np
import matplotlib.patches as mpatches
from scipy.signal import medfilt, medfilt2d
import cv2


def plot_segments_with_array_blocks(actual_h5_img_file, list_of_inds_to_plot, in_list_of_arrays=[], seg_num=0, border=4,
                                    height=20, img_width=61,
                                    color_numers_to_match=[0, 1, 2, 3, 4, 5], color_list=[0, .5, .2, .3, .75, .85],
                                    cmap_col='inferno', max_frames=40, min_frames=10):
    # apply max and min
    # in_list_of_arrays[0] needs to be the "true"" values
    if in_list_of_arrays == []:
        print('no input arrays, returning...')
        return
    color_dict = dict()
    cmap = cm.get_cmap(cmap_col)

    for i, k1 in enumerate(color_list):
        color_dict[i] = np.asarray(cmap(k1)[:-1]) * 255

    in_list_of_arrays = copy.deepcopy(in_list_of_arrays)

    # set/adjust size of the array
    inds = list(range(list_of_inds_to_plot[seg_num][0] - border, list_of_inds_to_plot[seg_num][-1] + 1 + border * 2))
    inds = inds[:max_frames]
    if len(inds) < min_frames:
        inds = np.arange(inds[0], inds[0] + min_frames)

    # get the image array with colored blocks
    for i, k in enumerate(in_list_of_arrays):
        k = k.astype(float)
        if i == 0:
            tmp1 = np.tile(np.repeat(k[inds], img_width, axis=0), (height, 1))
        else:
            tmp1 = np.vstack((tmp1, np.tile(np.repeat(k[inds], img_width, axis=0), (height, 1))))
    tmp1 = np.stack((tmp1,) * 3, axis=-1)

    for kk in color_numers_to_match:
        tmp3 = np.where(tmp1 == kk)
        for i1, i2 in zip(tmp3[0], tmp3[1]):
            tmp1[i1, i2, :] = color_dict[kk]

    tmp1 = tmp1.astype(int)
    with h5py.File(actual_h5_img_file, 'r') as h:
        tmp2 = image_tools.img_unstacker(h['images'][inds[0]:inds[-1] + 1], num_frames_wide=len(inds))
        print(tmp1.shape, tmp2.shape)
        tmp2 = np.vstack((tmp1, tmp2))
    return tmp2

def remap_array_to_color_channels(in_array, color_numers_to_match=None, color_list=[0, .5, .2, .3, .75, .85],
                                  cmap_col='inferno'):
    in_array = copy.deepcopy(in_array).astype(int)
    out_array = np.stack((in_array,) * 3, axis=-1)

    color_dict = dict()
    cmap = cm.get_cmap(cmap_col)
    if color_numers_to_match is None:
        color_numers_to_match = np.unique(in_array).astype(int)
        print(color_numers_to_match)

    for key, k1 in zip(color_numers_to_match, color_list):
        color_dict[key] = (np.asarray(cmap(k1)[:-1]) * 255).astype(int)
    for ii, kk in enumerate(color_numers_to_match):
        out_array[(in_array == kk).astype(bool)] = color_dict[kk]

    return out_array, color_dict


def foo_heatmap_with_critical_errors(real_bool, pred_bool, in_range, frame_nums, lines_thick=20, title_str='',
                                     figsize=(10, 10)):
    acc_percentage = ((pred_bool == real_bool) * 1).astype(float)
    acc_percentage[np.invert(in_range.astype(bool))] = np.nan
    acc_percentage = np.nanmean(acc_percentage)
    acc_percentage = str(np.round(acc_percentage * 100, 2)) + '%  '
    title_str = acc_percentage + title_str

    c_list = []
    for n in [2, 3, 4, 5, 8]:
        c_list.append(.0833 / 2 + n * .0833)

    max_ = np.max(frame_nums)
    x = np.zeros([len(frame_nums), int(max_)]) - 2

    d = real_bool - pred_bool
    d = d + (real_bool + pred_bool == 2) * 2  # TP = 2, TN = 0, FP = -1, FN = 1 ...... -2 pole out of range

    for i, (k1, k2) in enumerate(utils.loop_segments(frame_nums)):
        L = frame_nums[i]
        tmp1 = d[k1:k2]

        tmp1[in_range[k1:k2] == 0] = -2
        # in_range[k1:k2]
        x[i, :L] = tmp1

    x2, color_dict = remap_array_to_color_channels(x, color_numers_to_match=[0, 2, 1, -1, -2], color_list=c_list,
                                                   cmap_col='Paired')
    x2 = np.repeat(x2, lines_thick, axis=0)

    # get the color coded error type matrix
    a = analysis.error_analysis(real_bool, pred_bool, frame_num_array=frame_nums)
    d = copy.deepcopy(a.coded_array)
    d[d < 0] = -2
    d[d >= 4] = -2

    max_ = np.max(frame_nums)
    x = np.zeros([len(frame_nums), int(max_)]) - 2

    for i, (k1, k2) in enumerate(utils.loop_segments(frame_nums)):
        L = frame_nums[i]
        tmp1 = d[k1:k2]

        tmp1[in_range[k1:k2] == 0] = -2
        x[i, :L] = tmp1
    c_list = []
    for n in [1, 3, 4, 5, 8]:  # ['ghost', 'miss', 'join', 'split', nothing
        c_list.append(.1111 / 2 + n * .1111)
    x2_error_type, color_dict_error_type = remap_array_to_color_channels(x, color_numers_to_match=[0, 1, 2, 3, -2],
                                                                         color_list=c_list, cmap_col='Set1')
    print(np.nanmin(x2_error_type))
    x2_error_type = np.repeat(x2_error_type, lines_thick, axis=0)

    for i, (k1, k2) in enumerate(utils.loop_segments([10, 10] * len(
            frame_nums))):  # nan out certain regions so that we can leave those to be filled in with actual heatmap
        if (i % 2) != 0:
            x2_error_type[k1:k2] = color_dict_error_type[-2]

    x3 = copy.deepcopy(x2).astype(int)
    inds = x2_error_type != color_dict_error_type[-2]
    x3[inds] = x2_error_type[inds]
    plt.figure(figsize=figsize)
    plt.imshow(x3)

    # LEGEND
    all_labels = ['TN', 'TP', 'FN', 'FP', 'pole down']
    patches = []
    for i, ii in zip(color_dict, all_labels):
        c = color_dict[i] / 255
        patches.append(mpatches.Patch(color=c, label=ii))
    all_labels = ['ghost', 'miss', 'join', 'split']
    for i, ii in zip(color_dict_error_type, all_labels):
        c = color_dict_error_type[i] / 255
        patches.append(mpatches.Patch(color=c, label=ii))
    plt.legend(handles=patches, bbox_to_anchor=(1.02, 1), loc=2, borderaxespad=0, fontsize=15, prop={'size': 6})
    plt.title(title_str, fontsize=20)

    return x3

h5 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Finished_Sessions/Session1_FINISHED/Session1/AH1179X23052021x1_final_combined.h5'
h5 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Finished_Sessions/Session21_FINISHED/Session21/AH1184X08062021x21_final_combined.h5'
# h5_dst = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_FINISHED_MP4s_FINISHED/Session21/AH1184X08062021x21_final_combined.h5'
# keys = utils.print_h5_keys(h5, 1, 0)
# keys = utils.lister_it(keys, ['YHAT', 'labels', 'in_range', 'contacts_samson_curated_for_transfer_learning_220707'])
# for k in keys:
#     utils.copy_h5_key_to_another_h5(h5, h5_dst, k)

h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_FINISHED_MP4s_FINISHED/Session21/AH1184X08062021x21_final_combined.h5'

h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_FINISHED_MP4s_FINISHED/Session21/AH1184X08062021x21_final_combined_reduced.h5'
h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_V2_FINISHED/Session21/AH1184X08062021x21_final_combined.h5'


# h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Lilys_data/lilys_test_data_FINISHED/_final_combined.h5'

# h5 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Finished_Sessions/Session31/AH1184X15062021x31_final_combined.h5'

# tmp1 = image_tools.get_h5_key_and_concatenate(h5, 'full_file_names')
# utils.print_list_with_inds(utils.sort(utils.h5_string_switcher(tmp1)))

# bd = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session21_FINISHED_MP4s/Session21/'
# bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/PHILLIP/processing/P1_FINISHED_MP4s/AH0688/170808'
# utils.sort(utils.get_files(bd, '*.mp4'))
# bd = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/processing_119_FINISHED/'
# utils.sort(utils.get_files(bd, '*.h5'))
# h5 = '/Users/phil/Desktop/SAMSONS_TEST_SESSIONS/TESTING_SAMSONS_MP4S_FINISHED/AH1179X01062021x14_final_combined.h5'
####################################
def foo_rename_models(bd, num_combine=2, only_print=True):
    model_list = utils.get_files(bd, '*model.pkl')
    for k in model_list:
        new_name = '__'.join(k.split('/')[-num_combine:])
        print(new_name)
        print( os.path.dirname(k)+os.sep+new_name)
        if not only_print:
            os.rename(k, os.path.dirname(k)+os.sep+new_name)


# num_combine = 2
# bd = '/Users/phil/Desktop/model_saves/full_trial_101/'
# bd = '/Users/phil/Desktop/model_saves/samson_all_sessions_8_full_trials_each__weightedx2_V1'
# foo_rename_models(bd, num_combine=num_combine, only_print=True)
####################################a
model_list = utils.get_files('/Users/phil/Desktop/model_saves', '*.pkl')
model_list = utils.get_files('/Users/phil/Desktop/model_saves/full_trial_101/', '*model.pkl')
model_list = utils.get_files('/Users/phil/Desktop/model_saves/samson_all_sessions_8_full_trials_each__weightedx2_V1', '*model.pkl')
utils.print_list_with_inds(model_list)

# model_list = model_list[1]
# note the below I ran on 71 71 data and transfereed to this one file which is a 101 by 101 file
# data_file = '/Users/phil/Desktop/model_saves/data_saves/YHAT__samson_session_21_8_full_trial_with_optuna_weighted_TL_CNN_V1.pkl'
# y = utils.load_obj(data_file)
# utils.overwrite_h5_key(h5, 'YHAT__samson_session_21_8_full_trial_with_optuna_weighted_TL_CNN_V1', y)
# model_list = '/Users/phil/Desktop/model_saves/optuna_samson_all_sessions_8_full_trials_each__weightedx2_V1/optuna_samson_all_sessions_8_full_trials_each__weightedx2_V1_002_model.pkl'
# utils.foo_predict_mods(model_list, h5)
####################################
####################################
utils.lister_it(utils.print_h5_keys(h5, 1, 0), 'YHAT__')
utils.lister_it(utils.print_h5_keys(h5, 1, 0), 'YHAT2__')
# labels_key = 'YHAT__sess21_4perTrial_withOUT_sess1_OG_in_valData_V1'
# labels_key = 'YHAT__sess21_4perTrial_with_sess1_OG_NOTin_valData_V1'
labels_key = 'YHAT__sess21_4perTrial_with_sess1_OG_in_valData_V1'

# labels_key = 'YHAT__sess31_first__touch_2then6_4T_4V_with_sess1_OG_in_valData_V1'
labels_key = 'YHAT__sess21_first__touch_2then6_4T_4V_with_sess1_OG_in_valData_V1'
labels_key = 'YHAT__sess21_first__touch_4then4_alternating_with_sess1_OG_in_valData_FNisSUM_99x_with_noise_V4'
labels_key = 'YHAT__002_model'
# labels_key = 'YHAT__003_model_4_frames_form_each_trial_session_1'
# labels_key = 'YHAT__sess21_first__touch_2then6_4T_4V_with_sess1_OG_in_valData_V2'
labels_key = 'YHAT__sess21_101_by_101'
labels_key = 'YHAT2__LGBM_models_SPLIT_BY_TRIAL_NAN_mixed_samson_each_trial_with_16_from_session_21_only_at1300V6_003_model'


labels_key = 'YHAT__full_trial_101__samson_session_21_8_full_trial_weighted_V3__000_model'
# labels_key = 'YHAT__full_trial_101__samson_session_21_8_full_trial_V1__000_model'

# labels_key = 'YHAT3__samson_session_21_8_full_trial_with_optuna_weighted_71by71_V2'
# labels_key = 'YHAT__samson_all_sessions_8_full_trials_each__weightedx2_V1__000_model'

labels_key = 'YHAT__optuna_samson_all_sessions_8_full_trials_each__weightedx2_V1_002_model'

labels_key = 'YHAT__samson_session_21_8_full_trial_with_optuna_weighted_TL_CNN_V1'

# labels_key = 'YHAT__002_model'

pred_bool_temp = image_tools.get_h5_key_and_concatenate(h5, labels_key)
# contacts_samson_curated_for_transfer_learning_220707 = image_tools.get_h5_key_and_concatenate(h5, 'contacts_samson_curated_for_transfer_learning_220707')
# contacts_samson_curated_for_transfer_learning_220707 = image_tools.get_h5_key_and_concatenate(h5, 'samson_labels_matching')
contacts_samson_curated_for_transfer_learning_220707 = image_tools.get_h5_key_and_concatenate(h5, 'auto_transfer_labels_from_u_array')

labels = image_tools.get_h5_key_and_concatenate(h5, 'labels')
frame_nums = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
locations_x_y = np.asarray(image_tools.get_h5_key_and_concatenate(h5, 'locations_x_y'))
x_loca_all = locations_x_y[1500::3000, 0]
loc_x_sort_inds = np.argsort(x_loca_all)
################################################
""" SET THHE SOURCE OF THE LABELS YOU ARE DRAWING FROM MAKE SURE TO ADJUST MODEL IND ACCORDINGLY"""
threshold = .3
lines_thick = 20
kernel_size = 11
test_mood = False

if test_mood:### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP### TEMP
    pred_bool_smoothed = medfilt(copy.deepcopy(pred_bool_temp).flatten(), kernel_size=kernel_size)
    pred_bool_smoothed = (pred_bool_smoothed>threshold)*1

    real_bool = pred_bool_smoothed
    in_range = np.ones_like(pred_bool_temp)
else:
    pred_bool_smoothed = medfilt(copy.deepcopy(pred_bool_temp).flatten(), kernel_size=kernel_size)
    pred_bool_smoothed = (pred_bool_smoothed>threshold)*1

    in_range = 1*(contacts_samson_curated_for_transfer_learning_220707!=-1)

    real_bool = contacts_samson_curated_for_transfer_learning_220707
    real_bool[np.invert(in_range.astype(bool))] = -1
    real_bool = 1*(real_bool==1)



a = analysis.error_analysis(real_bool, pred_bool_smoothed, frame_nums)
ind = 0
x = a.all_errors_sorted[ind][0]
error_type = a.all_error_type_sorted[ind]

for i, (k1, k2) in enumerate(utils.loop_segments(frame_nums)):
    if k1 <= x < k2:
        print('trial num', i, error_type)
        y_line = i * lines_thick - lines_thick
        x_line = x - k1 - 1
        break

# x2 = foo_heatmap_with_critical_errors(real_bool, pred_bool_smoothed, in_range, frame_nums.astype(int), title_str='')


# tmp3 = plot_segments_with_array_blocks(h5, [[100]], in_list_of_arrays=[real_bool, pred_bool_smoothed],
#                                        seg_num=0,
#                                        color_numers_to_match=[0, 1], color_list=[0, .5], cmap_col='nipy_spectral',
#                                        max_frames=20, min_frames=20)

x2 = foo_heatmap_with_critical_errors(real_bool, pred_bool_smoothed, in_range, frame_nums.astype(int), title_str='')




tmp1 = []
plt.imshow(x2[0::20, :, :], interpolation='nearest', aspect='auto')
x4 = []
# fig, ax = plt.subplots(1, 2, sharex=True, gridspec_kw={'height_ratios': [10, 1]})
# fig, ax = plt.subplots(1, 2)
for k in np.flip(loc_x_sort_inds):
    kk = k*lines_thick
    tmp1 = x2[kk:kk+lines_thick, :, :]
    if not np.all(np.unique(tmp1) == [178, 202, 214]):
        # if k in [0, 1, 2, 3]:
        #     tmp1 = np.ones_like(tmp1)*20
        x4.append(tmp1)

x4 = np.vstack(x4)
plt.figure()
plt.title('sorted by x pole location')
plt.imshow(x4[0::20, :, :], interpolation='nearest', aspect='auto')

plt.figure()
plt.imshow(x2[0::20, :, :], interpolation='nearest', aspect='auto')

plt.plot(np.sort(x_loca_all))

# real_bool = real_bool[:3000*100]
# pred_bool_smoothed = pred_bool_smoothed[:3000*100]
# h5_images = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED_MP4s/Session1/image_frames.h5'
tmp3 = plot_segments_with_array_blocks(h5, [[10]], in_list_of_arrays=[real_bool, pred_bool_smoothed],
                                       seg_num=0,img_width=101,
                                       color_numers_to_match=[0, 1], color_list=[0, .5], cmap_col='nipy_spectral',
                                       max_frames=20, min_frames=20)



# tmp1 = image_tools.get_h5_key_and_concatenate(h5_images, 'labels')
# tmp1.shape

new_shape = (x2.shape[1], np.round((x2.shape[1] / tmp3.shape[1]) * tmp3.shape[0]).astype(int))
tmp3 = cv2.resize(tmp3.astype('float32'), new_shape)

x3 = np.vstack((x2, tmp3.astype(int)))

fig1 = plt.figure(figsize=[10, 8])
img_1 = plt.imshow(x3, interpolation='nearest', aspect='auto')
plt.tight_layout()

marker_point, = plt.plot(x_line, y_line, '.w', ms=8, alpha=.5) #mew=
marker_point.set_ydata(0)
marker_point.set_xdata(0)


def onclick(event):
    # global ix, iy, lines_thick, marker_point, real_bool_tmp, pred_bool_tmp, x2, img_1, frame_nums
    ix, iy = event.xdata - lines_thick, event.ydata - lines_thick

    print('x = %d, y = %d' % (ix, iy))

    # global coords
    # coords.append((ix, iy))
    #
    # if len(coords) == 2:
    #     fig.canvas.mpl_disconnect(cid)
    # foo(frame_nums, lines_thick, x)
    x3 = np.round(ix)
    trial = int(np.floor(iy / lines_thick))
    y3 = np.sum(frame_nums[:trial + 1])
    ind = int(x3 + y3)
    marker_point.set_ydata((trial * lines_thick)+lines_thick/2+lines_thick)
    marker_point.set_xdata(x3)
    print(x3, y3)

    # tmp3 = plot_segments_with_array_blocks(h5_file_IMG, [[ind]], in_list_of_arrays=[real_bool_tmp, pred_bool_tmp],
    #                                        seg_num=0,
    #                                        color_numers_to_match=[0, 1], color_list=[0, .5], cmap_col='nipy_spectral',
    #                                        max_frames=20, min_frames=20)
    img_width = 101
    tmp3 = plot_segments_with_array_blocks(h5, [[ind]], in_list_of_arrays=[real_bool, pred_bool_smoothed],
                                       seg_num=0,img_width=img_width,
                                       color_numers_to_match=[0, 1], color_list=[0, .5], cmap_col='nipy_spectral',
                                       max_frames=20, min_frames=20)
    new_shape = (x2.shape[1], np.round((x2.shape[1] / tmp3.shape[1]) * tmp3.shape[0]).astype(int))
    tmp3 = cv2.resize(tmp3.astype('float32'), new_shape)

    x3 = np.vstack((x2, tmp3.astype(int)))
    img_1.set_data(x3)
    # marker_point, = plt.plot(ix, iy-lines_thick, '+k', markersize=5)
    # return coords


fig = plt.gcf()
cid = fig.canvas.mpl_connect('button_press_event', onclick)

_ = plt.text(50, 1317, 'Real\nPred', fontsize=5, color='w')





plt.figure()
_ = plt.hist(pred_bool_temp, 100)


#
#
# a = analysis.error_analysis(1*(real_bool==1), 1*(pred_bool_smoothed==1), frame_num_array=frame_nums.astype(int))
# # pred_bool = copy.deepcopy(pred_bool_smoothed)
# # pred_bool[np.invert(in_range.astype(bool))] = -1
# x2 = foo_heatmap_with_critical_errors(real_bool, pred_bool_smoothed, in_range, frame_nums, title_str='median smoothed')


for i, (i1, i2) in enumerate(utils.loop_segments(frame_nums)):
    ii = i*lines_thick
    x = pd.Series(pred_bool_temp[i1:i2])
    tmp1 = np.argmax(x.rolling(window=10).std())
    plt.plot(tmp1,ii, '.k')


yhat = 'array of predicitons output fomr model'
from scipy.signal import medfilt
yhat_smoothed = medfilt(copy.deepcopy(yhat), 7)
yhat_smoothed = 1*(yhat_smoothed>.5)
utils.overwrite_h5_key()









tmp1 = np.linspace(0, 1, 311)*1000
tmp2 = np.round(tmp1)
plt.plot(tmp2, '.')




# from whacc import touch_gui
# h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_V2_FINISHED/Session21/AH1184X08062021x21_final_combined.h5'
# touch_gui(h5, 'samson_labels_matching')


utils.print_h5_keys(h5)
tmp1 = utils.search_sequence_numpy(real_bool, np.asarray([1, 1, 0, 1, 1]))
trial_nums_and_frame_nums = image_tools.get_h5_key_and_concatenate(h5, 'trial_nums_and_frame_nums')
full_file_names = utils.h5_string_switcher(image_tools.get_h5_key_and_concatenate(h5, 'full_file_names'))

for k in tmp1:
    print('trial --> ', full_file_names[k//3000])
    print('frame--> ', 2+k%3000)
    print('____')


max_val_stack = image_tools.get_h5_key_and_concatenate(h5, 'max_val_stack')
max_val_stack

#
# h5 = '/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_V2_FINISHED/Session21/AH1184X08062021x21_final_combined.h5'
# with h5py.File(h5, 'r') as h:
#     for k in utils.print_h5_keys(h5, 1, 0):
#         try:
#             print(h[k].shape)
#         except:
#             try:
#                 print(len(h[k]))
#             except:
#                 pass
#
#         pass
#         print(k)
#         print('________')
d = utils.h5_to_dict('/Volumes/GoogleDrive-114825029448473821206/My Drive/Session21_V4_FINISHED/Session21/AH1184X08062021x21_final_to_combine_1_to_5_of_275.h5', None])
